/*
  AntiRight
  (c) 2002-2004 Jeffrey Bedard
  jefbed@e-list.net
 
  This file is part of AntiRight.

  AntiRight is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  AntiRight is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with AntiRight; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* FIXME:  Make this portable, especially to NetBSD.  */
#include "../../lib/library.h"
#include <fcntl.h>
#include <sys/ioctl.h>

#ifdef HAVE_LINUX_CDROM_H
#include <linux/cdrom.h>
#endif

/* CD-ROM file descriptor id.  */
int arcd_file_descriptor;
/* Track list structure.  */
#ifdef HAVE_LINUX_CDROM_H
struct cdrom_tochdr arcd_track_list;
#endif /* HAVE_LINUX_CDROM_H  */
/* Track index structure.  */
#ifdef HAVE_LINUX_CDROM_H
struct cdrom_ti arcd_tracks;
#endif /* HAVE_LINUX_CDROM_H  */

void
arcd_setup_file_descriptor()
{
  char* cdrom_string;
  cdrom_string=antiright_pipe_read("echo -n `ACE -r cdrom`");    
  close(arcd_file_descriptor);
  arcd_file_descriptor=open(cdrom_string, O_RDONLY);
  free(cdrom_string);
}
void
initialize_cdrom()
{
  arcd_setup_file_descriptor();
#ifdef HAVE_LINUX_CDROM_H
  ioctl(arcd_file_descriptor, CDROMREADTOCHDR, &arcd_track_list);
  arcd_tracks.cdti_trk0=arcd_track_list.cdth_trk0;
  arcd_tracks.cdti_trk1=arcd_track_list.cdth_trk1;
#endif /* HAVE_LINUX_CDROM_H  */
}
void
arcd_play()
{
#ifdef HAVE_LINUX_CDROM_H
  ioctl(arcd_file_descriptor, CDROMPLAYTRKIND, &arcd_tracks);
#endif /* HAVE_LINUX_CDROM_H  */
}
void
arcd_initialized_command(int command)
{
  initialize_cdrom();
  ioctl(arcd_file_descriptor, command);
}
void
arcd_usage()  
{
  puts("Usage:  arcd [-PrsT] [-t TRACK]");
  exit(1);
}
void
arcd_start_play_at_track(int* counter_int,
			 char** argv)
{
  Boolean track_not_before_first_track, track_not_after_last_track;

  (*counter_int)++;
  initialize_cdrom();
#ifdef HAVE_LINUX_CDROM_H
  track_not_before_first_track = (atoi(argv[(*counter_int)]) 
				  >= arcd_track_list.cdth_trk0);
  track_not_after_last_track = (atoi(argv[(*counter_int)]) 
				<= arcd_track_list.cdth_trk1);
  
  if(track_not_before_first_track & track_not_after_last_track)
    {
      arcd_tracks.cdti_trk0=atoi(argv[(*counter_int)]);
      arcd_play();
    }
  else
    {
      arcd_usage();
    }
#endif /* HAVE_LINUX_CDROM_H  */
}

void
arcd_print_number_of_tracks()
{
  /* This prints the number of tracks on the current CD.  */
  initialize_cdrom();
#ifdef HAVE_LINUX_CDROM_H
  printf("%d", arcd_track_list.cdth_trk1);
#endif /* HAVE_LINUX_CDROM_H  */
  fflush(stdout);
}
int
main(int argc, char** argv)    
{
  int counter_int;  
  for(counter_int=1; counter_int < argc; counter_int++)
    if(argv[counter_int][0]=='-')
      {
	switch(argv[counter_int][1])
	  {
	  case 'P': /* This pauses play.  */
#ifdef HAVE_LINUX_CDROM_H
	    arcd_initialized_command(CDROMPAUSE);
#endif /* HAVE_LINUX_CDROM_H  */
	    break;
	  case 's': /* This stops the cdrom.  */
#ifdef HAVE_LINUX_CDROM_H
	    arcd_initialized_command(CDROMSTOP);
#endif /* HAVE_LINUX_CDROM_H  */
	    break;
	  case 'r': /* This resumes play.  */
#ifdef HAVE_LINUX_CDROM_H
	    arcd_initialized_command(CDROMRESUME);
#endif /* HAVE_LINUX_CDROM_H  */
	    break;
	  case 't': /* This starts play from a particular track.  */
	    arcd_start_play_at_track(&counter_int, argv);
	    break;
	  case 'T': 
	    arcd_print_number_of_tracks();
	    break;
	  default:
	    arcd_usage();
	    break;
	  }
      }

  if(argc==1)
    arcd_usage();
  
  return(0);
}
