/*
  AntiRight
  (c) 2002-2004 Jeffrey Bedard
  jefbed@e-list.net
 
  This file is part of AntiRight.

  AntiRight is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  AntiRight is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with AntiRight; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "arshell.h"

#if XmVERSION > 1 & XmREVISION > 1
#include <Xm/ColorS.h>

#else

Widget arcolor_red_widget;
Widget arcolor_green_widget;
Widget arcolor_blue_widget;
Widget arcolor_preview_widget;

void
arcolor_allocate_color(XColor *color)
{
  Colormap colormap;

  /* These options are set often enough, so set them in this generic
     function.  */
  color->flags=DoRed|DoGreen|DoBlue;
  
  colormap=DefaultColormap(XtDisplay(arcolor_preview_widget), 
			   DefaultScreen
			   (XtDisplay(arcolor_preview_widget)));
  
  
  XAllocColor(XtDisplay(arcolor_preview_widget),
	      colormap,
	      color);
}

void
arcolor_update_preview()
{
  XColor color;
  char* label_string;
  
  XmScaleGetValue(arcolor_red_widget, (int*)&color.red); 
  XmScaleGetValue(arcolor_green_widget, (int*)&color.green);
  XmScaleGetValue(arcolor_blue_widget, (int*)&color.blue);

  asprintf(&label_string, "rgb:%x/%x/%x", color.red, color.green, color.blue);
  antiright_label_string(arcolor_preview_widget, label_string);
  free(label_string);

  color.red=color.red*256;
  color.green=color.green*256;
  color.blue=color.blue*256;

  arcolor_allocate_color(&color);

  XtVaSetValues(arcolor_preview_widget,
		XmNbackground, color.pixel,
		NULL);
}

void
antiright_color_scale_cb()
{
  arcolor_update_preview();
}

Widget 
arcolor_scale(Widget parent_widget, 
	      char *title_string)
{
  XmString title_motif_string;
  Widget scale_widget;
  
  title_motif_string=
    XmStringCreateLocalized(title_string);
  antiright_set(XmNorientation, XmHORIZONTAL);
  antiright_set(XmNmaximum, 255);
  antiright_set(XmNwidth, 255);
  antiright_set(XmNvalue, 0);
  antiright_set(XmNshowValue, True);
  antiright_set(XmNtitleString, (XtArgVal)title_motif_string);
  scale_widget=XmCreateScale(parent_widget,
			     "scale_widget",
			     antiright_arguments,
			     antiright_number_arguments);
  XmStringFree(title_motif_string);
  XtManageChild(scale_widget);

  XtAddCallback(scale_widget,
		XmNdragCallback,
		antiright_color_scale_cb,
		NULL);
  return(scale_widget);
}


#endif /* #if XmVERSION > 1 & XmREVISION > 1  */

void
arcolor_select_cb(Widget parent_widget,
		  XtPointer client_data,
		  XtPointer call_data)
{ 
#if XmVERSION > 1 & XmREVISION > 1
  char* color_string;
  ARCLARG;
  antiright_set(XmNcolorName, (XtArgVal)&color_string);
  XtGetValues((Widget)client_data, 
	      antiright_arguments, antiright_number_arguments);
  puts(color_string);
  free(color_string);
#else
  int red_int;
  int green_int;
  int blue_int;
  
  XmScaleGetValue(arcolor_red_widget, &red_int);
  XmScaleGetValue(arcolor_green_widget, &green_int);
  XmScaleGetValue(arcolor_blue_widget, &blue_int);

  /* This is the means by which data from this program is
     obtained.  */
  printf("rgb:%x/%x/%x", red_int, green_int, blue_int);
  fflush(stdout);
#endif

  parent_widget=NULL;
  call_data=NULL;

  exit(0);
}

Widget
arshell_color_widget(Widget parent_widget)
{
  Widget arcolor_widget;
#if XmVERSION > 1 & XmREVISION > 1
  Widget color_selector_widget;
#else
  Widget preview_frame_widget;
#endif /* XmVERSION > 1 & XmREVISION > 1  */

  ARCLARG;
  arcolor_widget=antiright_form(parent_widget);

#if XmVERSION > 1 & XmREVISION > 1
  ARCLARG;
  antiright_form_positions(0, 95, 0, 100);
  color_selector_widget=XmCreateColorSelector(arcolor_widget,
					      "color_selector_widget",
					      antiright_arguments,
					      antiright_number_arguments);
  XtManageChild(color_selector_widget);
  ARCLARG;
  antiright_form_positions(95, 100, 0, 100);
  antiright_button(arcolor_widget, "Select", 
		   arcolor_select_cb, color_selector_widget);
#else
  ARCLARG;
  antiright_form_positions(2, 66, 2, 34);
  preview_frame_widget=antiright_frame(arcolor_widget, XmSHADOW_IN);
  ARCLARG;
  arcolor_preview_widget=antiright_label(preview_frame_widget, "Color");
  ARCLARG;
  antiright_form_positions(66, 98, 2, 34);
  antiright_button(arcolor_widget,"Select",arcolor_select_cb,NULL);
  ARCLARG;
  antiright_form_positions(2, 34, 34, 98);
  arcolor_red_widget=arcolor_scale(arcolor_widget, "Red");
  ARCLARG;
  antiright_form_positions(34, 66, 34, 98);
  arcolor_green_widget=arcolor_scale(arcolor_widget, "Green");
  ARCLARG;
  antiright_form_positions(66, 98, 34, 98);
  arcolor_blue_widget=arcolor_scale(arcolor_widget, "Blue");
  arcolor_update_preview();
  puts("method");
  
#endif /* XmVERSION > 1 & XmREVISION > 1 */

  return(arcolor_widget);
}
